/*********************************************************************
 *
 *                Thermo Sensor
 *
 *********************************************************************
 * FileName:        temperature.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include "Compiler.h"
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "temperature.h"

#if defined(__18CXX)
    #if defined(PICDEM_FS_USB)
        #include <spi.h>
    #else
        #include "SoftwareSPI.h"
    #endif
#endif

/** V A R I A B L E S ********************************************************/
#pragma udata
BOOL old_sw;
WORD_VAL temperature;       // Raw data format
char tempString[10];        // Buffer for storing data in ASCII format

/** P R I V A T E  P R O T O T Y P E S ***************************************/

/** D E C L A R A T I O N S **************************************************/
#pragma code
/******************************************************************************
 * Function:        void InitTempSensor(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Initializes SPI interface & chip select line
 *
 * Note:            None
 *****************************************************************************/
void InitTempSensor(void)
{
    #if defined(PICDEM_FS_USB)
    cs_temp_sensor = 1;
    tris_cs_temp_sensor = OUTPUT_PIN;
    OpenSPI(SPI_FOSC_64, MODE_11, SMPMID);
    
    // Initialize readable values - default to room temperature
    temperature.Val = 0x0C87;       // 25 degree celsius
    UpdateCelsiusASCII();
    
    #elif defined(LOW_PIN_COUNT_USB_DEVELOPMENT_KIT)
        //Initialize SW SPI interface to TC72
        ANSEL = 0;
        ANSELH = 0;
        LOpenSWSPI ();
        LSetCESWSPI();        //clear CS pin
        LWriteSWSPI( 0x80 );
        LWriteSWSPI( 0x00 );
        LClearCESWSPI();        //clear CS pin

 	#elif defined(__C30__) || defined(__C32__)
        #if defined(__C30__)
	   		AD1CHS = 0x4;           //MUXA uses AN4
    		AD1PCFGLbits.PCFG4 = 0;
            AD1CON3 = 0x0002;
            AD1CON2 = 0;
            AD1CON1 = 0x8000; // turn ADC ON
       #else
            AD1PCFGbits.PCFG4 = 0;
            AD1CON1 = 0x0000; // SAMP bit = 0 ends sampling ...
            // and starts converting
            AD1CHS = 0x00040000; // Connect RB4/AN4 as CH0 input ..
            // in this example RB2/AN2 is the input
            AD1CSSL = 0;
            AD1CON3 = 0x0002; // Manual Sample, Tad = internal 6 TPB
            AD1CON2 = 0;
            AD1CON1SET = 0x8000; // turn ADC ON
        #endif
 	
    #endif    
}//end InitTempSensor

/******************************************************************************
 * Function:        void AcquireTemperature(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        None
 *
 * Note:            None
 *****************************************************************************/
BOOL AcquireTemperature(void)
{
    #if defined(PICDEM_FS_USB)
		//The PICDEM FS USB Demo Board uses a TC77 (13 bit) temperature sensor and
		//communicates with it through the SPI interface.
		cs_temp_sensor = 0;
		temperature.v[1] = ReadSPI();
		temperature.v[0] = ReadSPI();
		cs_temp_sensor = 1;

		if(temperature.bits.b2 == 0)
			return FALSE;

	#elif defined(__C30__) || defined(__C32__)
		
		// The Explorer 16 Demo Board uses a TC1047 analog-out temperature sensor
		// The 10-bit measurement is converted to the 13-bit TC77 format so we can
		// copy/use the same routine in UpdateCelsiusASCII   
		
		//Create temp variables to store the conversion data
		float temp;
		int counter;
				
		//get ready to sample the A/D
        #if defined(__C30__)
			// Get an ADC sample
			AD1CON1bits.SAMP = 1;           //Start sampling
			for(counter=0;counter<1000;counter++); //Sample delay, conversion start automatically
			AD1CON1bits.SAMP = 0;           //Start sampling
//			for(counter=0;counter<1000;counter++); //Sample delay, conversion start automatically
			while(!AD1CON1bits.DONE);       //Wait for conversion to complete

       #else
            AD1CON1SET = 0x0002; // start sampling ...
            for(temp=0;temp<1000;temp++); //Sample delay, conversion start automatically
            AD1CON1CLR = 0x0002; // start Converting
            while (!(AD1CON1 & 0x0001));// conversion done?
        #endif
		//convert the results to a float
		temp = (float)ADC1BUF0;

		// voltage = A2D_reading * 3.3v / 1024
		temp *= 3.3;
		temp /= 1024;

		// align to 0C (subtracting -.65v)
		temp -= .55;

		//convert to TC77 style output and store to temperature
		temp *= 12800;
		temperature.Val = (WORD)temp;
		temperature.Val |= 0x7;

    #elif defined(LOW_PIN_COUNT_USB_DEVELOPMENT_KIT)
        LSetCESWSPI();        //clear CS pin
		LWriteSWSPI( 0x02 );
		temperature.v[1] = LWriteSWSPI( 0 );
		temperature.v[0] = LWriteSWSPI( 0 );
		LClearCESWSPI();
		temperature.Val>>=1;
        return TRUE;
	#else
		#error Unknown temperature acquire configuration.  See AcquireTemperature function in __FILE__
    #endif
    return TRUE;
}//end AcquireTemperature

/******************************************************************************
 * Function:        void UpdateCelsiusASCII(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine converts data output to ASCII string
 *
 * Note:            None
 *****************************************************************************/
void UpdateCelsiusASCII(void)
{
    WORD_VAL temp;
    BYTE i;
        
    temp.Val = temperature.Val >> 3;
    
    if(temp.bits.b12 == 0)
    {
        temp.byte.HB &= 0x1F;
        tempString[0] = '+';
    }
    else
    {
        temp.byte.HB |= 0xE0;
        tempString[0] = '-';
        temp.Val = temp.Val ^ 0xFFFF;   // Negate
        temp.Val++;
    }//end if

    temp.Val = (temp.Val*10U) >> 4;     // Turn into celsius xx.x
    
    /* Populate string */
    for(i=4;i>0;i--)
    {
        tempString[i] = (((char)(temp.Val % 10)) & 0x0F) | 0x30;
        temp.Val /= 10;
    }//end for
    
    /* Turn leading zeros into spaces */
    if(tempString[1] == '0')
    {
        tempString[1] = ' ';
        if(tempString[2] == '0')
            tempString[2] = ' ';
    }//end if
    
    /* Adjust decimal digit */
    tempString[5] = tempString[4];
    tempString[4]='.';
    tempString[6]=0xF8;     // Degree symbol
    tempString[7]='C';
    tempString[8]='\r';
    tempString[9]=0x00;     // Null-Terminated
    
}//end UpdateCelsiusASCII

/*******************************************************************************
 * Function:        BOOL SwitchIsPressed(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          Returns TRUE if sw was pressed
 *                  Returns FALSE if sw was not pressed
 *
 * Side Effects:    None
 *
 * Overview:        This function is used to check sw switch.
 *
 * Note:            None
 ******************************************************************************/
BOOL SwitchIsPressed(void)
{
    if(sw != old_sw)
    {
        old_sw = sw;                  // Save new value
        if(sw == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end SwitchIsPressed

/** EOF temperature.c ********************************************************/
